/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class PLine extends Vertex implements Sketchable, VertexListener
{
    ArrayList vertices;

    public PLine(double x, double y) 
    {
	super(x, y);
	super.sketchedObjects.add(this);
	vertices = new ArrayList();
	DEFAULT_COLOR = Color.red.darker().darker();
	changeColor(DEFAULT_COLOR);
	init();
    }

    private void init()
    {
	vertices.clear();
	vertices.add(this);
	addPoint(x, y);
    }

    public boolean inBound(double x, double y)
    {
	boolean result = super.inBound(x, y);
	return result;
    }

	// FIXME:  WHY IS THE FOLLOWING NECESSARY?
	// FIXME:  ERASE HIGHLIGHTING
	// FIXME:  ERASE REMOTE
    public void draw(Graphics g, DrawingArea drawArea) 
    {
	super.draw(g, drawArea);

	Vertex vprev = null;
	for(int c = 0; c < vertices.size(); c++) {
	    Vertex v = (Vertex) vertices.get(c);
	    //v.draw(g, drawArea);

	    if(vprev != null) {
		Vertex start = drawArea.transform(vprev.x, vprev.y);
		Vertex end   = drawArea.transform(v.x, v.y);
		g.drawLine((int) start.x, (int) start.y, (int) end.x, (int) end.y);
	    }
	    vprev = v;
	}
    }

    public String toString()
    {
	String result = "PLine [";
	result += super.toString();
	for(int c = 1; c < vertices.size(); c++) {
	    Vertex v = (Vertex) vertices.get(c);
	    result += " "+v.toString(null);
	}
	result += "]";
	return result;
    }

    public Vertex addPoint(double x, double y) 
    {
	System.out.println("PLine:: addPoint");
	Vertex v = new Vertex(x, y);
	v.setVertexListener(this);
	v.DEFAULT_COLOR = Color.red.darker().darker();
	v.changeColor(v.DEFAULT_COLOR);
	vertices.add(v);
	super.sketchedObjects.add(v);
	return v;
    }

    public void changeColor(Color c)
    {
	super.changeColor(null, c);

	if(vertices == null)
	    return;

	Iterator it = vertices.iterator();
	while(it.hasNext()) {
	    Vertex v = (Vertex) it.next();
	    v.changeColor(null, c); // non recursive to prevent stack overflow
	}
    }

    public boolean remove()
    {
	Iterator it = vertices.iterator();
	while(it.hasNext()) {
	    Object o = it.next();
	    super.sketchedObjects.remove(o);
	}
	vertices.clear();
	super.sketchedObjects.remove(this);
	return true;
    }
    
    public Object clone()
    {
	Util.assert_nonjava(false, "Shape::clone not implemented");
	return null;
    }

    public Vertex updateCoord(double x, double y)
    {
	Util.assert_nonjava(vertices.size() > 1, "PLine::updateCoord");
	Vertex v = (Vertex) vertices.get(vertices.size() - 1);
	v.mouseDragged(x, y, null, 0);
	return v;
    }
}
