/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package samplemon;

import gnu.getopt.*;
import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;

import javax.swing.*;
import java.awt.*;
import java.util.*;

import cricketdaemon.UnitConv;

/**
 *
 * @author Allen Miu
 */
public class SampleMonApp extends Thread
    implements Callback, Runnable
{
    // screen size
    static final int WIDTH  = 1000;
    static final int HEIGHT  = 300;

    private long timeLastUpdate = 0;
    private static final long REFRESH_PERIOD = 200;

    private JPanel panel;
    private Subplot plotter;
    Broker cricket;

    public static boolean useMetric = true;

    public SampleMonApp(Broker c, double history, double maxy, boolean sm)
    {
	panel = new JPanel();
	GridBagLayout gridbag = new GridBagLayout();
        panel.setLayout(gridbag);	

	plotter = new SampleMonSubplot(panel, 0, history,
					  "Beacon Samples", 
					  "Distance (cm)", 
					  "Time (s)", maxy, sm);
	cricket = c;
	BitSet mask = new BitSet();
	mask.set(Broker.BEACONSTAT);
	mask.set(Broker.CALCBEACONDIST);
	cricket.register(this, mask);

	start();
    }

    public JPanel getPanel() { return panel; }

    public void setHistory(double history) { plotter.setHistory(history); }

    synchronized public void callback(CricketData data, BitSet mask)
    {
	if(mask.get(Broker.BEACONSTAT)) {
	    Iterator it = data.getBeaconsHeard().iterator();
	    while(it.hasNext()) {
		BeaconRecord br = (BeaconRecord) it.next();
		Iterator its = br.distStat.samples.iterator();
		//System.out.println(br.space+" "+br.distStat.samples.size());
		
		while(its.hasNext()) {
		    Sample s = (Sample) its.next();
		    //System.out.println("\t"+ s);
		    plotter.process(br.space, s.time, 
				    UnitConv.cricketDistToCM(s.dist));
		}
	    }
	}
	flush();
    }

    synchronized public void run()
    {
	while(true) {
	    try {
		wait(REFRESH_PERIOD);
	    } catch (InterruptedException ie) {
		
	    }
	    flush();
	}
    }

    synchronized private void flush()
    {
	long curTime = System.currentTimeMillis();
	if((curTime - timeLastUpdate) >= REFRESH_PERIOD) {
	    plotter.flush();
	    timeLastUpdate = curTime;
	}
    }

    public static void usage()
    {
	System.err.println("Usage: java samplemon.SampleMonApp [-H history_in_ms][-S][-y max_y_axis_cm]\n");
	System.exit(1);
    }

    public static void main(String[] args) 
    {
	Getopt g = new Getopt("SampleMonApp", args, "Sy:H:");
	double history = 4000; // in miliis
	double maxy    = 500; // 5 meters
	boolean scanMode = true;

	int c;
	try {
	    while((c = g.getopt()) != -1) {
		switch(c) {
		case 'S':
		    scanMode = false;
		    break;
		case 'y':
		    maxy    = Double.parseDouble(g.getOptarg());
		    break;
		case 'H':
		    history = Double.parseDouble(g.getOptarg());
		    break;
		default:
		    usage();
		}
	    }
	}
	catch (Exception e) {
	    usage();
	}

	Broker s = new ServerBroker();
	SampleMonApp sma = new SampleMonApp(s, history, maxy, scanMode);
	CustomFrame frame = new CustomFrame("Sample Monitor", WIDTH, HEIGHT);
	frame.getContentPane().add(sma.getPanel());
	frame.finishInit();

	s.start();
	try { sma.join(); } catch (InterruptedException ie) { }
	s.interrupt();
    }
}
